package com.aitrainer.api.controller.diet

import com.aitrainer.api.model.diet.Diet
import com.aitrainer.api.repository.diet.DietRepository
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*
import java.util.*

@RestController
@RequestMapping("/api")
class DietController(private val dietRepository: DietRepository) {

    @PostMapping("/diet")
    fun insert(@RequestBody diet: Diet): ResponseEntity<*> {
        return ResponseEntity.ok().body(dietRepository.save(diet))
    }

    @PostMapping("/diet/{id}")
    fun update(@PathVariable(value = "id") id: Long, @RequestBody diet: Diet): ResponseEntity<Diet> {
        val existingDiet  = dietRepository.findByDietId(id) ?: return ResponseEntity.notFound().build()

        val updatedDiet: Diet = existingDiet.copy(
            dietId = diet.dietId,
            dietUserId = diet.dietUserId,
            dietText = diet.dietText,
            startDate = diet.startDate
        )
        diet.meals.forEach {
            it.diet = diet
            updatedDiet.meals.add(it)
        }
        return ResponseEntity.ok().body(dietRepository.save(updatedDiet))
    }

    @GetMapping("/diet/{dietUserId}")
    fun getByDietUserId(@PathVariable dietUserId: Long): ResponseEntity<List<Diet>> {
        val list = dietRepository.findByDietUserId(dietUserId)
        return if (list.isEmpty()) ResponseEntity.notFound().build() else
            ResponseEntity.ok().body(list)
    }
}