package com.aitrainer.api.openai

import com.aallam.openai.client.OpenAI
import com.aallam.openai.api.completion.CompletionRequest
import com.aallam.openai.api.completion.TextCompletion
import com.aallam.openai.api.logging.LogLevel
import com.aallam.openai.api.model.Model
import com.aallam.openai.api.model.ModelId
import com.aallam.openai.client.OpenAIConfig
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext
import java.util.Properties

class OpenAIService(private val modelName: String?, private val temperature: Double?) {


    private var openAI: OpenAI? = null
    var model: Model? = null
    private val properties = Properties()

    init {
        val inputStream = ClassLoader.getSystemResourceAsStream("application.properties")
        properties.load(inputStream)
        inputStream?.close()
    }

    private var modelId: ModelId? = null
    private suspend fun connect(modelName: String) {
        val config = OpenAIConfig(
            token = properties.getProperty("openai.key"),
            logLevel = LogLevel.All
        )
        openAI = OpenAI(config)
        modelId = ModelId(modelName)
        model = openAI!!.model(modelId!!)

    }

    suspend fun completion(question: String): String {
        return withContext(Dispatchers.IO) {
            var realModelName = "text-davinci-003"
            if ( modelName != null) {
                realModelName = modelName
            }
            var realTemperature = 0.1
            if ( temperature != null ) {
                realTemperature = temperature
            }
            if (openAI == null) {
                connect(realModelName)
            }
            println("OpenAI Question: $question")
            val completionRequest = CompletionRequest(
                model = modelId!!,
                prompt = question,
                //echo = true,
                maxTokens = 2048 - question.length,
                temperature=realTemperature,
            )
            val completion: TextCompletion = openAI!!.completion(completionRequest)

            val result = completion.choices[0].text
            result
        }
    }

    suspend fun getModels(): MutableList<String> {
        return withContext(Dispatchers.IO)  {
            if (openAI == null) {
                openAI = OpenAI(token = properties.getProperty("openai.key"))
            }

            val list: MutableList<String> = mutableListOf()
            openAI!!.models().forEach {
                println(it)
                list.add(it.id.id)
            }

            return@withContext list
        }
    }
}

